-- Some basic variables
TITAN_HONORMARKS_ID = "HonorMarks";
TITAN_HONORMARKS_VERSION = "2.1.6";
TITAN_HONORMARKS_BUTTON_LABEL = "Marks: ";
TITAN_HONORMARKS_BUTTON_ICON = "Interface\\Icons\\Ability_DualWield";

-- colors
TEXT_COLOR_WHITE = "|cffffffff";
TEXT_COLOR_GOLD = "|cffCEA208";
TEXT_COLOR_RED = "|cffff0000";
TEXT_COLOR_GREEN = "|cff00ff00";
TEXT_COLOR_ORANGE = "|cffFF9900";

-- Initializing constants
MARK_DEBUG = false; -- set 1 for debugging purposes
MARK_GREEN_TRESH = 20;
MARK_ORANGE_TRESH = 97;
MARK_RED_TRESH = 100;
MARK_HONOR_GREEN_TRESH = 22500;
MARK_HONOR_ORANGE_TRESH = 45000;
MARK_HONOR_RED_TRESH = 100000;
HONORMARKS_MAX_PLAYER_BAGS = 4;
HONORMARKS_MAX_BANK_BAGS = 7;
HONORMARKS_SEPERATOR = "/";

bank_opened = false;

-- OnLoad
function TitanPanelHonorMarksButton_OnLoad( self )
	-- Register Plugin with Titan
	this.registry = { 
		id = TITAN_HONORMARKS_ID,
		menuText = TITAN_HONORMARKS_ID .. " v" .. TITAN_HONORMARKS_VERSION, 
		buttonTextFunction = "TitanPanelHonorMarksButton_GetButtonText", 
		tooltipTitle = TITAN_HONORMARKS_ID .. " v" .. TITAN_HONORMARKS_VERSION, 
		tooltipTextFunction = "TitanPanelHonorMarksButton_GetTooltipText", 
		icon = TITAN_HONORMARKS_BUTTON_ICON,
		iconWidth = 16,
		savedVariables = {
			ShowAV = 1,
			ShowAB = 1,
			ShowWSG = 1,
			ShowEOTS = 1,
			ShowArena = 1,
			ShowHonor = 1,
			ShowIcon = 1,
			ShowLabelText = 1,
			ShowColoredText = TITAN_NIL,
		}
	};

	-- Init the database
	TitanPanelHonorMarks_DBInit();
	
	-- Register events
	TitanPanelHonorMarks_RegisterEvents();

	-- Registering Slash Command
	SLASH_HONORMARKS1 = "/honormarks";
	SlashCmdList["HONORMARKS"] = TitanPanelHonorMarks_SlashHandler;
end

--------------------------------------------------------------------------------------------------
-- Register & Unregister Event functions.
--------------------------------------------------------------------------------------------------
-- RegisterEvent
function TitanPanelHonorMarks_RegisterEvents()
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("HONOR_CURRENCY_UPDATE");
end

-- UnregisterEvent
function TitanPanelHonorMarks_UnregisterEvents()
	this:UnregisterEvent("VARIABLES_LOADED");
	this:UnRegisterEvent("HONOR_CURRENCY_UPDATE");
end

-- SlashHandler
function TitanPanelHonorMarks_SlashHandler(command)
	local cmd=strlower(command);
	if(cmd=="debug") then
		if(MARK_DEBUG) then
			MARK_DEBUG = false;
		else
			MARK_DEBUG = true;
		end
		TitanPanelHonorMarks_Text("Debugging: "..boolformat(MARK_DEBUG));
	end
end

-- OnEvent
function TitanPanelHonorMarks_OnEvent(self, event)
	-- debugging
	if(MARK_DEBUG) then
		str = event;
		if(arg1 ~= nil) then
			str = str .. " - arg1: " .. stringformat(arg1);
		end;
		if(arg2 ~= nil) then
			str = str .. " - arg2: " .. stringformat(arg2);
		end;
		TitanPanelHonorMarks_Debug(str);
	end 

	if(event == "VARIABLES_LOADED") then
	--	TitanPanelHonorMarks_Text("Event:"..tostring(event));
		TitanPanelHonorMarks_Text(TitanPlugins[TITAN_HONORMARKS_ID].menuText.." loaded."); 
	end
	if(event == "HONOR_CURRENCY_UPDATE" ) then
	--	TitanPanelHonorMarks_Text("Event:"..tostring(event));
		TitanPanelHonorMarks_UpdateCount();
	end
	TitanPanelButton_UpdateButton(TITAN_HONORMARKS_ID);
end

-- Titan Functions
function TitanPanelHonorMarksButton_GetButtonText(id)
	-- init
	local av_count, ab_count, wsg_count, eots_count, honor_currency, arena;
	
	-- Count Honor Marks
	honor_currency, av_count, ab_count, wsg_count, eots_count, arena = TitanPanelHonorMarks_GetCount(); 
	--TitanPanelHonorMarks_Text("[GBT] Honor:"..tostring(honor_currency).." AV:"..tostring(av_count).." AB:"..tostring(ab_count).." WSG:"..tostring(wsg_count).." EOTS:"..tostring(eots_count));
	if ( av_count == nil ) then
		av_count = 0;
	end
	if ( ab_count == nil ) then
		ab_count = 0;
	end
	if ( wsg_count == nil ) then
		wsg_count = 0;
	end
	if ( eots_count == nil ) then
		eots_count = 0;
	end
	if ( arena == nil ) then 
		arena = 0;
	end 


	-- build button
	local buttonText = "";
	local buttonArray = {};
	if(TitanGetVar(TITAN_HONORMARKS_ID,"ShowHonor")) then 
		table.insert(buttonArray, HONORMARKS_HONOR_LABEL..":"..TitanPanelHonorMarks_ColorHonor(honor_currency));
	end
	if(TitanGetVar(TITAN_HONORMARKS_ID,"ShowAV")) then
		table.insert(buttonArray, "AV:"..TitanPanelHonorMarks_ColorNumber(av_count));
	end
	if(TitanGetVar(TITAN_HONORMARKS_ID,"ShowAB")) then
		table.insert(buttonArray, "AB:"..TitanPanelHonorMarks_ColorNumber(ab_count));
	end
	if(TitanGetVar(TITAN_HONORMARKS_ID,"ShowWSG")) then
		table.insert(buttonArray, "WSG:"..TitanPanelHonorMarks_ColorNumber(wsg_count));
	end
	if(TitanGetVar(TITAN_HONORMARKS_ID,"ShowEOTS")) then 
		table.insert(buttonArray, HONORMARKS_EOTS_LABEL..":"..TitanPanelHonorMarks_ColorNumber(eots_count));
	end
	if(TitanGetVar(TITAN_HONORMARKS_ID,"ShowArena")) then 
		table.insert(buttonArray, HONORMARKS_ARENA..":"..TitanPanelHonorMarks_ColorNumber(arena));
	end
	buttonText = table.concat(buttonArray, HONORMARKS_SEPERATOR);	
	buttonText = table.concat(buttonArray, HONORMARKS_SEPERATOR);	

	-- return
	return TITAN_HONORMARKS_BUTTON_LABEL, buttonText;
end

function TitanPanelHonorMarksButton_GetTooltipText()
	-- init
	local av_count, ab_count, wsg_count, eots_count, honor_currency, arena;
	
	-- Count Honor Marks
	honor_currency, av_count, ab_count, wsg_count, eots_count, arena = TitanPanelHonorMarks_GetCount();
	--TitanPanelHonorMarks_Text("[GTT] Honor:"..tostring(honor_currency).." AV:"..tostring(av_count).." AB:"..tostring(ab_count).." WSG:"..tostring(wsg_count).." EOTS:"..tostring(eots_count));
	if ( av_count == nil ) then
		av_count = 0;
	end
	if ( ab_count == nil ) then
		ab_count = 0;
	end
	if ( wsg_count == nil ) then
		wsg_count = 0;
	end
	if ( eots_count == nil ) then
		eots_count = 0;
	end
	if ( arena == nil ) then 
		arena = 0;
	end 

	-- Build Tooltip
	local tooltipText = "";
	tooltipText = tooltipText..TEXT_COLOR_GREEN..HONORMARKS_HONOR_CURRENCY.."\t"..TitanPanelHonorMarks_ColorHonor(honor_currency).."|r\n";
	tooltipText = tooltipText..TEXT_COLOR_GREEN..HONORMARKS_AV_MARK.."\t"..TitanPanelHonorMarks_ColorNumber(av_count).."|r\n";
	tooltipText = tooltipText..TEXT_COLOR_GREEN..HONORMARKS_AB_MARK.."\t"..TitanPanelHonorMarks_ColorNumber(ab_count).."|r\n";
	tooltipText = tooltipText..TEXT_COLOR_GREEN..HONORMARKS_WSG_MARK.."\t"..TitanPanelHonorMarks_ColorNumber(wsg_count).."|r\n";
	tooltipText = tooltipText..TEXT_COLOR_GREEN..HONORMARKS_EOTS_MARK.."\t"..TitanPanelHonorMarks_ColorNumber(eots_count).."|r\n";
	tooltipText = tooltipText..TEXT_COLOR_GREEN..HONORMARKS_ARENA.."\t"..TitanPanelHonorMarks_ColorNumber(arena).."|r\n";
	
	-- return
	return tooltipText;
end

function TitanPanelRightClickMenu_PrepareHonorMarksMenu()
	-- Menu Title
	TitanPanelRightClickMenu_AddTitle(TitanPlugins[TITAN_HONORMARKS_ID].menuText);
	
	-- Honor
	local info = {};
	info.text = HONORMARKS_MENU_SHOWHONOR;
	info.func = TitanPanelHonorMarks_ShowHonor;
	info.checked = TitanGetVar(TITAN_HONORMARKS_ID,"ShowHonor");
	UIDropDownMenu_AddButton(info);
	-- AV
	local info = {};
	info.text = HONORMARKS_MENU_SHOWAV;
	info.func = TitanPanelHonorMarks_ShowAV;
	info.checked = TitanGetVar(TITAN_HONORMARKS_ID,"ShowAV");
	UIDropDownMenu_AddButton(info);
	-- AB
	local info = {};
	info.text = HONORMARKS_MENU_SHOWAB;
	info.func = TitanPanelHonorMarks_ShowAB;
	info.checked = TitanGetVar(TITAN_HONORMARKS_ID,"ShowAB");
	UIDropDownMenu_AddButton(info);
	-- WSG
	local info = {};
	info.text = HONORMARKS_MENU_SHOWWSG;
	info.func = TitanPanelHonorMarks_ShowWSG;
	info.checked = TitanGetVar(TITAN_HONORMARKS_ID,"ShowWSG");
	UIDropDownMenu_AddButton(info);
	-- EOTS
	local info = {};
	info.text = HONORMARKS_MENU_SHOWEOTS;
	info.func = TitanPanelHonorMarks_ShowEOTS;
	info.checked = TitanGetVar(TITAN_HONORMARKS_ID,"ShowEOTS");
	UIDropDownMenu_AddButton(info);
	-- ARENA
	local info = {};
	info.text = HONORMARKS_MENU_SHOWARENA;
	info.func = TitanPanelHonorMarks_ShowArena;
	info.checked = TitanGetVar(TITAN_HONORMARKS_ID,"ShowArena");
	UIDropDownMenu_AddButton(info);
	
	-- General Titan Options
	TitanPanelRightClickMenu_AddSpacer();	
	TitanPanelRightClickMenu_AddToggleIcon(TITAN_HONORMARKS_ID);
	TitanPanelRightClickMenu_AddToggleLabelText(TITAN_HONORMARKS_ID);
	TitanPanelRightClickMenu_AddToggleColoredText(TITAN_HONORMARKS_ID);
	
	TitanPanelRightClickMenu_AddSpacer();	
	TitanPanelRightClickMenu_AddCommand(TITAN_PANEL_MENU_HIDE, TITAN_HONORMARKS_ID, TITAN_PANEL_MENU_FUNC_HIDE);
end

-- Titan Menu Toggles
function TitanPanelHonorMarks_ShowAV()
	TitanToggleVar(TITAN_HONORMARKS_ID, "ShowAV");
	TitanPanelButton_UpdateButton(TITAN_HONORMARKS_ID);
end

function TitanPanelHonorMarks_ShowAB()
	TitanToggleVar(TITAN_HONORMARKS_ID, "ShowAB");
	TitanPanelButton_UpdateButton(TITAN_HONORMARKS_ID);
end

function TitanPanelHonorMarks_ShowWSG()
	TitanToggleVar(TITAN_HONORMARKS_ID, "ShowWSG");
	TitanPanelButton_UpdateButton(TITAN_HONORMARKS_ID);
end

function TitanPanelHonorMarks_ShowEOTS()
	TitanToggleVar(TITAN_HONORMARKS_ID, "ShowEOTS");
	TitanPanelButton_UpdateButton(TITAN_HONORMARKS_ID);
end

function TitanPanelHonorMarks_ShowHonor()
	TitanToggleVar(TITAN_HONORMARKS_ID, "ShowHonor");
	TitanPanelButton_UpdateButton(TITAN_HONORMARKS_ID);
end

function TitanPanelHonorMarks_ShowArena()
	TitanToggleVar(TITAN_HONORMARKS_ID, "ShowArena");
	TitanPanelButton_UpdateButton(TITAN_HONORMARKS_ID);
end

function TitanPanelHonorMarks_SetCount( honor, av_count, ab_count, wsg_count, eots_count, arena )
        -- Store Data in DB
	TitanPanelHonorMarks_SetVar("HONORMARKS_REC_HONOR", honor); 
	TitanPanelHonorMarks_SetVar("HONORMARKS_REC_AV", av_count); 
	TitanPanelHonorMarks_SetVar("HONORMARKS_REC_AB", ab_count); 
	TitanPanelHonorMarks_SetVar("HONORMARKS_REC_WSG", wsg_count); 
	TitanPanelHonorMarks_SetVar("HONORMARKS_REC_EOTS", eots_count);
	TitanPanelHonorMarks_SetVar("HONORMARKS_REC_ARENA", arena);
end

function TitanPanelHonorMarks_GetCount()
	-- Pulling data from DB
	return TitanPanelHonorMarks_GetVar("HONORMARKS_REC_HONOR"), TitanPanelHonorMarks_GetVar("HONORMARKS_REC_AV"),
		TitanPanelHonorMarks_GetVar("HONORMARKS_REC_AB"),
		TitanPanelHonorMarks_GetVar("HONORMARKS_REC_WSG"),
		TitanPanelHonorMarks_GetVar("HONORMARKS_REC_EOTS"),
		TitanPanelHonorMarks_GetVar("HONORMARKS_REC_ARENA");
end

-- Counting Functions
function TitanPanelHonorMarks_UpdateCount( wasClosed, headerIndex )
	local honor = GetHonorCurrency();
	local av_count = 0;
	local ab_count = 0;
	local wsg_count = 0;
	local eots_count = 0;
	local arena = 0;
	local closeHeader = wasClosed;
	local closeHeaderIndex = headerIndex;
	
	local name, myindex, isHeader, isExpanded, isUnused, isWatched, count, cCount, PVPIndex;
	cCount = GetCurrencyListSize();
	for index=1, cCount do 
		name, isHeader, isExpanded, isUnused, isWatched, count = GetCurrencyListInfo(index);

		if ( isHeader and name == "Player vs. Player" ) then
			TitanPanelHonorMarks_Debug("Found PVP Header: "..name);
			if (not isExpanded ) then 
				TitanPanelHonorMarks_Debug("PVP Header: "..name.." ["..index.."] is closed.  Will open");
				ExpandCurrencyList(index, 1);
				honor, av_count, ab_count, wsg_count, eots_count = TitanPanelHonorMarks_UpdateCount(true, index);
			end
		end

		if ( not isHeader ) then
			if ( name == HONORMARKS_AV_MARK ) then
				av_count = count;
			end
			if ( name == HONORMARKS_AB_MARK ) then
				ab_count = count;
			end
			if ( name == HONORMARKS_WSG_MARK ) then
				wsg_count = count;
			end
			if ( name == HONORMARKS_EOTS_MARK ) then
				eots_count = count;
			end
			if ( name == HONORMARKS_ARENA ) then
				arena = count;
			end

		end 
		myindex=index
	end 
	TitanPanelHonorMarks_Debug("Exited Loop: Index at "..tostring(myindex));

	if ( closeHeader ) then
		TitanPanelHonorMarks_Debug("Closing CurrencyList["..tostring(closeHeaderIndex).."]: Index at "..tostring(myindex));
		ExpandCurrencyList(closeHeaderIndex, 0); 
	end 
	
	TitanPanelHonorMarks_SetCount( honor, av_count, ab_count, wsg_count, eots_count, arena );

	return honor, av_count, ab_count, wsg_count, eots_count, arena;
end

-- Color
function TitanPanelHonorMarks_ColorNumber(count)
	local color = TEXT_COLOR_GOLD;
	if(TitanGetVar(TITAN_HONORMARKS_ID, "ShowColoredText")) then
		if(count < MARK_GREEN_TRESH) then
			color = TEXT_COLOR_WHITE;
		elseif(count < MARK_ORANGE_TRESH) then
			color = TEXT_COLOR_GREEN;
		elseif(count < MARK_RED_TRESH) then
			color = TEXT_COLOR_ORANGE;
		else
			color = TEXT_COLOR_RED;
		end
	end
	return color..count.."|r";
end

function TitanPanelHonorMarks_ColorHonor(count)
	local color = TEXT_COLOR_GOLD;
	if(TitanGetVar(TITAN_HONORMARKS_ID, "ShowColoredText")) then
		if(count < MARK_HONOR_GREEN_TRESH) then
			color = TEXT_COLOR_WHITE;
		elseif(count < MARK_HONOR_ORANGE_TRESH) then
			color = TEXT_COLOR_GREEN;
		elseif(count < MARK_HONOR_RED_TRESH) then
			color = TEXT_COLOR_ORANGE;
		else
			color = TEXT_COLOR_RED;
		end
	end
	return color..count.."|r";
end

-- Useful
function TitanPanelHonorMarks_NameFromLink(link)
	return GetItemInfo(link);
end

function TitanPanelHonorMarks_NameFromBagSlot(bag, slot)
	return TitanPanelHonorMarks_NameFromLink(GetContainerItemLink(bag, slot));
end

-- Alert Messages
function TitanPanelHonorMarks_Text(msg)
	if ( msg == nil or msg == false ) then
		msg = "nil";
	end
	if ( DEFAULT_CHAT_FRAME) then
		DEFAULT_CHAT_FRAME:AddMessage("*** HonorMarks: "..msg, 1.0, 0.22, 0);
	end
end

function TitanPanelHonorMarks_Debug(msg)
	if ( msg == nil or msg == false ) then
		msg = "nil";
	end
	if ( DEFAULT_CHAT_FRAME and MARK_DEBUG ) then
		DEFAULT_CHAT_FRAME:AddMessage("*** HonorMarks Debug: "..msg, 1.0, 0.22, 0);
	end
end

function boolformat(boolean)
	if(boolean) then
		return "true";
	else
		return "false";
	end
end

function stringformat(string)
	if(type(string) == "boolean") then
		return boolformat(string);
	end
	return tostring(string);
end

-- Database Functions
function TitanPanelHonorMarks_DBInit()
	HONORMARKS_COUNTREC = {};
	HONORMARKS_COUNTREC["HONORMARKS_REC_HONOR"] = 0;
	HONORMARKS_COUNTREC["HONORMARKS_REC_AV"] = 0;
	HONORMARKS_COUNTREC["HONORMARKS_REC_AB"] = 0;
	HONORMARKS_COUNTREC["HONORMARKS_REC_WSG"] = 0;
	HONORMARKS_COUNTREC["HONORMARKS_REC_EOTS"] = 0;
	HONORMARKS_COUNTREC["HONORMARKS_REC_ARENA"] = 0;
end

function TitanPanelHonorMarks_SetVar(var, value)
	HONORMARKS_COUNTREC[var] = value;
end

function TitanPanelHonorMarks_GetVar(var)
	return HONORMARKS_COUNTREC[var];
end
